from math import *
from random import *
from effect import Effect
from constants import *

# EffectHandler
# EffectHandler keeps track of all Effect particles in the game.
# It is used to create effects such as explosions and camera shake.

class EffectHandler():
    def __init__(self):
        self.bricks = []
        self.flares = []
        self.clouds = []
        self.shakeAmount = 0
        
        self.createClouds()
    
    def update(self):
        for effects in [self.bricks, self.flares, self.clouds]:
            for i in range(len(effects)-1, -1, -1):
                e = effects[i]
                e.update()
                if not e.isAlive():
                    effects.remove(e)
            self.shakeAmount *= 0.9
            
        for c in self.clouds:
            if c.x > screen_width:
                c.x = -400

    def createExplosion(self, x, y, acc):
        r = int(2 + acc / 50.0)
	for i in range(int(1.5*r)):
	    direction = 2 * pi * random()
	    speed = r / 10.0 * (1 + 4*random())
	    rad = 30 * (1 + random())
	    pX = x + rad * sin(direction)
	    pY = y + rad * cos(direction)
	    size = 0.9 + 0.2 * random()
	    drop = (1 + random()) / 1000.0

	    self.bricks.append(Effect('brick', pX, pY, size, drop, direction, speed))

    def createFlare(self, physics, lJet, rJet):
        pos = physics.getPlayerCenter()
        angle = physics.player.angle
        px = physics.thrust * sin(angle)
        py = physics.thrust * cos(angle)
        fake = 55
        jet = [lJet, rJet]
        dx = []
        dx.append(pos[0] + physics.tdx * cos(-angle) + fake * sin(angle))
        dx.append(pos[0] - physics.tdx * cos(-angle) + fake * sin(angle))
        dy = []
        dy.append(pos[1] + physics.tdx * sin(-angle) + fake * cos(angle))
        dy.append(pos[1] - physics.tdx * sin(-angle) + fake * cos(angle))
        
        r = 1
	for i in range(r):
            for e in range(2):
                direction = angle + pi/15.0 * (-0.5 + random())
                speed = 6.0 * (1 + 2*random())
                size = 1.0 + 0.7 * random()
                drop = (1 + 4 * random()) / 30.0
                x = dx[e] + 10 * (-0.5 + random())
                y = dy[e] + 10 * (-0.5 + random())
                
                if jet[e]:
                    self.flares.append(Effect('flare', x, y, size, drop, direction, speed))
            
    def createClouds(self):
        def farAway(x,y):
            for c in self.clouds:
                if ((c.x-x)**2 + (c.y-y)**2)**0.5 < 200:
                    return False
            return True
        
        k = 20
        w = int(screen_width + bgscale * (wbcw - screen_width))
        h = int(screen_height + bgscale * (wbch - screen_height))
        print w, h
        for j in range(k*(w / screen_width)):
            i = 0
            while True:
                i += 1
                x = randint(-400, w)
                y = randint(-200, h-500)
                if not farAway(x,y) and i < 50:
                    continue
                direction = pi/2.0
                speed = 0.05 * (1 + 3*random())
                self.clouds.append(Effect('cloud', x, y, 1, 0, direction, speed))
                break

    def addShake(self, amount):
	if amount/2.0 > self.shakeAmount:
	    self.shakeAmount = amount/2.0

    def getBricks(self):
	return self.bricks
        
    def getFlares(self):
	return self.flares
        
    def getClouds(self):
	return self.clouds

    def getShake(self):
	return self.shake
