import pygame, sys

# -- IO --
# Collects user key- and mouse-input and stores it in maps

class IO():
    # All stored user input
    text = ''
    keys = {}
    keys_pressed = {}
    keys_released = {}
    mouse = {'x': 0, 'y': 0, 'pre_x': 0, 'pre_y': 0}
    mouse_pressed = {}
    mouse_released = {}
    
    # Main pygame event handler
    def update(self):
        # Reset
        for e in self.keys_pressed:
            self.keys_pressed[e] = False
        for e in self.keys_released:
            self.keys_released[e] = False
        for e in self.mouse_pressed:
            self.mouse_pressed[e] = False
        for e in self.mouse_released:
            self.mouse_released[e] = False
        self.mouse['pre_x'] = self.mouse['x']
        self.mouse['pre_y'] = self.mouse['y']
        
        # Loop through stack of events
        pygame.event.pump()
        for event in pygame.event.get():
            
            # Quit-button
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
                
            # Key pressed
            if event.type == pygame.KEYDOWN:
                try: self.keys[event.key]
                except: self.keys[event.key] = False
                if not self.keys[event.key]:
                    self.keys_pressed[event.key] = True
                    if event.key >= 97 and event.key <= 122:
                        self.text += pygame.key.name(event.key)
                    if event.key == 32 and self.text:
                        self.text += ' '
                    if event.key == 8:
                        self.text = self.text[0:-1]
                self.keys[event.key] = True
                
            # Key released
            if event.type == pygame.KEYUP:
                try: self.keys[event.key]
                except: self.keys[event.key] = True
                if self.keys[event.key]:
                    self.keys_released[event.key] = True
                self.keys[event.key] = False
                
            # Mouse pressed
            if event.type == pygame.MOUSEBUTTONDOWN:
                try: self.mouse[event.button]
                except: self.mouse[event.button] = False
                if not self.mouse[event.button]:
                    self.mouse_pressed[event.button] = True
                self.mouse[event.button] = True
                
            # Mouse released
            if event.type == pygame.MOUSEBUTTONUP:
                try: self.mouse[event.button]
                except: self.mouse[event.button] = False
                if self.mouse[event.button]:
                    self.mouse_released[event.button] = True
                self.mouse[event.button] = False
                
            # Mouse move
            if event.type == pygame.MOUSEMOTION:
                (self.mouse['x'], self.mouse['y']) = event.pos
            
    # Keys
    
    # Check if key is held down
    def check_key(self, key):
        try: return self.keys[key]
        except: return False
    
    # Check if key is pressed
    def check_key_pressed(self, key):
        try: return self.keys_pressed[key]
        except: return False

    # Check if key is released
    def check_key_released(self, key):
        try: return self.keys_released[key]
        except: return False
    

    # Mouse
    
    # Check if mousebutton is held down
    def check_mouse(self, button):
        try: return self.mouse[button]
        except: return False
    
    # Check if mousebutton is pressed
    def check_mouse_pressed(self, button):
        try: return self.mouse_pressed[button]
        except: return False

    # Check if mousebutton is released
    def check_mouse_released(self, button):
        try: return self.mouse_released[button]
        except: return False
    
    # Get mouse position - tuple (x,y)
    def get_mouse(self):
        try: return (self.mouse['x'], self.mouse['y'])
        except: return False
    
    # Get mouse relative movement - tuple(rx,ry)
    def get_mouse_previous(self):
        try: return (self.mouse['pre_x'], self.mouse['pre_y'])
        except: return False
    
    
    # User written text (A-Z + space)
    
    # Reset written text
    def reset_text(self, text=''):
        self.text = text

    # Get written text
    def get_text(self):
        return self.text


