# 2014 GameJam
# Mans - Programming
# Raymond - Art
# Jesper - Sound

import pygame, sys
from math import *
from random import *

from graphics import Graphics
from io import IO
from sound import Sound
from player import Player
from physics import Physics
from effectHandler import EffectHandler
from constants import *

# Initialize pygame
pygame.init()

# Initialize template classes
graphics = Graphics(screen_width, screen_height, 50, 'CastleLander')
io = IO()
sound = Sound()
physics = Physics()
effectHandler = EffectHandler()


# Game objects
player = Player(220, 140)
fragility = 150
nextFragility = 400
damageStage = 1
deathStage = 5
isBoosting = False
isDead = False

graphics.createBackground(effectHandler.getClouds())
sound.loop_sound('song')

def restart():
    physics.__init__()
    effectHandler.__init__()
    graphics.createBackground(effectHandler.getClouds())
    global damageStage
    damageStage = 1
    global isDead
    isDead = False
    sound.set_volume('song', 0.5)


# -- Game loop --
while True:
    
    # Basic
    graphics.update()
    io.update()
    player.update()
    physics.update(not isDead)
    effectHandler.update()
    
    if io.check_key_pressed(k_esc):
        pygame.quit()
        sys.exit()
    
    # Controls
    
    if io.check_key_pressed(ord('r')):
        restart()
    
    lJet = False
    rJet = False
    if damageStage < deathStage:
        if io.check_key(k_up) and not (io.check_key(k_left) or io.check_key(k_right)):
            lJet = True
            rJet = True
        if io.check_key(k_left):
            lJet = True
        if io.check_key(k_right):
            rJet = True
    effectHandler.createFlare(physics, lJet, rJet)
    physics.boost(rJet, lJet)
    if not isBoosting and (lJet or rJet):
        sound.loop_sound('boost')
        isBoosting = True
    if isBoosting and not (lJet or rJet):
        sound.stop_sound('boost')
        isBoosting = False
    
    if physics.getAcc() > fragility:
        pos = physics.getPlayerCenter()
        acc = physics.getAcc()
        effectHandler.createExplosion(pos[0], pos[1], acc)
        if acc > fragility:
            sound.play_sound('hurt{}'.format(randint(1,3)))
            damageStage += 1
            acc -= fragility
            while acc > nextFragility:
                acc -= nextFragility
                damageStage += 1
    if damageStage >= deathStage:
        if not isDead:
            sound.play_sound('death')
            sound.set_volume('song', 0)
            isDead = True
        physics.removePlayer()
        damageStage = deathStage
    if physics.isOutside():
        restart()
    if physics.checkClaimPlatform():
        print 'win!'
        sound.play_sound('win')
    
    # Draw
    if damageStage < deathStage:
        graphics.cam_follow_player(physics.player)
    
    print
    graphics.draw_background()
    #graphics.draw_clouds(effectHandler.getClouds())
    graphics.draw_platforms(physics.platforms, physics.claimedPlatforms)
    graphics.draw_hills(physics.hills)
    graphics.draw_flares(effectHandler.getFlares())
    graphics.draw_bricks(effectHandler.getBricks())
    graphics.draw_physics(physics, damageStage)
    graphics.draw_fps()

