import pygame
from pygame.locals import *
from pygame.color import *
import pymunk
from pymunk import Vec2d
from math import *
from constants import *
import time
from random import *


def timing(f):
    def wrap(*args):
        time1 = time.time()
        ret = f(*args)
        time2 = time.time()
        print '%s function took %0.3f ms' % (f.func_name, (time2-time1)*1000.0)
        return ret
    return wrap


class Physics:
    
    def flipyv(self, v):
        return v[0], screen_height - v[1]
    
    def get_collision(self, space, arb):
        for c in arb.contacts:
            r = max(3, int(abs(c.distance*50)))
            p = self.flipyv(c.position)
            self.collisions.append((map(int, p), r))
        
    def __init__(self):
        self.running = True
        self.drawing = True
        self.lJet = 0
        self.rJet = 0
        self.thrust = 10
        self.tdx = 52
        self.tdy = -15
        
        ### Init pymunk and create space
        self.space = pymunk.Space()
        self.space.gravity = (0.0, -900.0)
        self.space.add_collision_handler(0, 0, None, None, self.get_collision, None)
        self.collisions = []
        
        
        self.platforms = []
        self.claimedPlatforms = list(self.platforms)
        self.hills = []
        
        for i in range(100):
            self.createHill(-30 + 300*i, 50, 3)
        
        def platformhill(x, y):
            self.createHill(x, y, 2)
            self.createPlatform(x+30, y+40)
        
        """
        x = 0
        while x < 10000:
            x += randint(300,600)
            y = randint(0,310)
            platformhill(x, y)
        """
        self.createPlatform(1200, 100)
        self.createPlatform(450, 100)
        
        self.createHill(-50, 200, 2)
        self.createHill(700, 400, 1)
        
        
        self.createPlayer(100,215)
    
    def createHill(self, x, y, hType):
        body = pymunk.Body()
        body.position = Vec2d(x, y)
        points = []
        shapes = []
        if hType == 1:
            points.append((35, -450))
            points.append((30, -180))
            points.append((40, -140))
            points.append((75, -80))
            points.append((120, -60))
            points.append((180, -50))
            points.append((250, -60))
            points.append((295, -90))
            points.append((325, -140))
            points.append((330, -180))
            points.append((340, -450))
        if hType == 2:
            points.append((30, -310))
            points.append((30, -35))
            points.append((315, -35))
            points.append((315, -310))
        if hType == 3:
            points.append((20, -35))
            points.append((325, -35))
        for i in range(len(points)-1):
            shapes.append(pymunk.Segment(body, points[i], points[i+1], .0))
        for s in shapes:
            s.friction = 0.9
            s.elasticity = 0.6
            self.space.add(s)
        self.hills.append((body, hType))
        
    def createPlatform(self, x, y):
        body = pymunk.Body()
        body.position = Vec2d(x, y)
        k = 0.75
        points = [(5, -125), (50, -6), (330, -6), (375, -125)]
        points = [map(int, [f*k for f in e]) for e in points]
        shapes = []
        for i in range(len(points)-1):
            shapes.append(pymunk.Segment(body, points[i], points[i+1], .0))
        for s in shapes:
            s.friction = 0.8
            s.elasticity = 0.5
            self.space.add(s)
        self.platforms.append(body)
    
    def createPlayer(self, x, y):
        size= 50
        points = [(-size*1.2, -size), (-size*1.4, size*0.7), (0,size*1.1), (size*1.4,size*0.7), (size*1.2, -size)]
        mass = 1.0
        moment = pymunk.moment_for_poly(mass, points, (0,0))
        body = pymunk.Body(mass, moment)
        body.position = Vec2d(x,y)
        shape = pymunk.Poly(body, points, (0,0))
        shape.friction = 0.7
        shape.elasticity = 0.3
        self.space.add(body,shape)
        self.player = body
        self.preVel = list(self.player.velocity)
    
    @timing
    def update(self, alive):
        px = self.thrust * sin(-self.player.angle)
        py = self.thrust * cos(-self.player.angle)
        if self.lJet:
            self.player.apply_impulse((px,py), (-self.tdx,self.tdy))
        if self.rJet:
            self.player.apply_impulse((px,py), (self.tdx,self.tdy))
        self.preVel = list(self.player.velocity)
        
        if self.player.position[1] > wbch - 50 and alive:
            self.player.apply_impulse((0,-30), (0,0))
        if self.player.position[0] < wbcx + 50 and alive:
            self.player.apply_impulse((30,0), (0,0))
        if self.player.position[0] > wbcw - 50 and alive:
            self.player.apply_impulse((-30,0), (0,0))
        
        self.collisions = []
        steps = 3
        dt = 1.0/120.0/steps            
        for x in range(steps):
            self.space.step(dt)
       
    def boost(self, left, right):
        self.lJet = left
        self.rJet = right

    def getPlayerCenter(self):
        return self.flipyv(self.player.position)

    def getAcc(self):
        relVel = (self.player.velocity - self.preVel)
        acc = sum(map(lambda x: x**2, relVel))**0.5
        return acc

    def removePlayer(self):
        self.space.remove(self.player.shapes)
        
    def isOutside(self):
        return (self.player.position[1] < wbcy - 80)

    def checkClaimPlatform(self):
        for p in self.platforms:
            if map(int, self.player.velocity) == [0,0]:
                k = 0.75
                x1 = int(p.position[0])
                y1 = int(p.position[1])
                x2 = int(self.player.position[0])
                y2 = int(self.player.position[1])
                dx1 = int(k*55)
                dx2 = int(k*320)
                dy = int(45)
                if x1+dx1 < x2 < x1+dx2 and y1+dy == y2:
                    if p not in self.claimedPlatforms:
                        self.claimedPlatforms.append(p)
                        return True
        return False
